import geompy
import math

import salome, SMESH
import smesh
import StdMeshers


# define four corner points
dw = 0.5
X1 = 10.0
Y1 = 8.0
p1 = geompy.MakeVertex(0.0, 0.0, 0.0)
p2 = geompy.MakeVertex(X1,  0.0, 0.0)
p3 = geompy.MakeVertex(X1,  Y1,  0.0)
p4 = geompy.MakeVertex(0.0, Y1,  0.0)

dwl = 0.7  # 'width' of long springs
dws = 0.5  # 'width' of short springs
dx2a = 2.0
dx2b = dx2a+dwl
dy2a = 2.0
dy2b = dy2a+dws
ps1  = geompy.MakeVertex(dw,      dw,     0.0)
ps2 =  geompy.MakeVertex(dx2a,    dw,     0.0)
ps3 =  geompy.MakeVertex(dx2b,    dw,     0.0)
ps4 =  geompy.MakeVertex(X1-dw,   dw,     0.0)
#second line
ps5 = geompy.MakeVertex(X1-dw,  dy2a,   0.0)
ps6 = geompy.MakeVertex(X1-dw,  dy2b,   0.0)
ps7 = geompy.MakeVertex(X1-dw,  Y1-dw,  0.0)
# third line
ps8  = geompy.MakeVertex(X1-dx2a,   Y1-dw,     0.0)
ps9  = geompy.MakeVertex(X1-dx2b,   Y1-dw,     0.0)
ps10 = geompy.MakeVertex(dw,        Y1-dw,     0.0)
# fourth line
ps11 = geompy.MakeVertex(dw,  Y1-dy2a,   0.0)
ps12 = geompy.MakeVertex(dw,  Y1-dy2b,   0.0)

dw2 = 1.5
dx2a = 6.0
dx2b = dx2a+dwl  # 'width' of long springs
dy2a = 5.0
dy2b = dy2a+dws  # 'width' of short springs
pt1  = geompy.MakeVertex(dw2,     dw2,     0.0)
pt2 =  geompy.MakeVertex(dx2a,    dw2,     0.0)
pt3 =  geompy.MakeVertex(dx2b,    dw2,     0.0)
pt4 =  geompy.MakeVertex(X1-dw2,   dw2,    0.0)

pt5 = geompy.MakeVertex(X1-dw2,  dy2a,     0.0)
pt6 = geompy.MakeVertex(X1-dw2,  dy2b,     0.0)
pt7 = geompy.MakeVertex(X1-dw2,  Y1-dw2,   0.0)

pt8  = geompy.MakeVertex(X1-dx2a,   Y1-dw2,  0.0)
pt9  = geompy.MakeVertex(X1-dx2b,   Y1-dw2,  0.0)
pt10 = geompy.MakeVertex(dw2,        Y1-dw2, 0.0)

pt11 = geompy.MakeVertex(dw2,  Y1-dy2a,   0.0)
pt12 = geompy.MakeVertex(dw2,  Y1-dy2b,   0.0)

dwi = 0.8
pi1 =  geompy.MakeVertex(dw2+dwi,  dw2+dwi,        0.0)
pi2 =  geompy.MakeVertex(X1-dw2-dwi,dw2+dwi,       0.0)
pi3 =  geompy.MakeVertex(X1-dw2-dwi, Y1-dw2-dwi,   0.0)
pi4 =  geompy.MakeVertex(dw2+dwi,  Y1-dw2-dwi,     0.0)



# define lines of outer rectangle outside
L1a = geompy.MakeLineTwoPnt(p1, p2)
L2a = geompy.MakeLineTwoPnt(p2, p3)
L3a = geompy.MakeLineTwoPnt(p3, p4)
L4a = geompy.MakeLineTwoPnt(p4, p1)

# define lines of outer rectangle inside
L1b = geompy.MakeLineTwoPnt(ps1, ps2)
L3b = geompy.MakeLineTwoPnt(ps3, ps4)
L4b = geompy.MakeLineTwoPnt(ps4, ps5)
L6b = geompy.MakeLineTwoPnt(ps6, ps7)
L7b = geompy.MakeLineTwoPnt(ps7, ps8)
L9b = geompy.MakeLineTwoPnt(ps9, ps10)
L10b = geompy.MakeLineTwoPnt(ps10, ps11)
L12b = geompy.MakeLineTwoPnt(ps12, ps1)

L1c = geompy.MakeLineTwoPnt(pt1, pt2)
L3c = geompy.MakeLineTwoPnt(pt3, pt4)
L4c = geompy.MakeLineTwoPnt(pt4, pt5)
L6c = geompy.MakeLineTwoPnt(pt6, pt7)
L7c = geompy.MakeLineTwoPnt(pt7, pt8)
L9c = geompy.MakeLineTwoPnt(pt9, pt10)
L10c = geompy.MakeLineTwoPnt(pt10, pt11)
L12c = geompy.MakeLineTwoPnt(pt12, pt1)


# define spring elements
Lsp1a = geompy.MakeLineTwoPnt(ps2, pt2)
Lsp2a = geompy.MakeLineTwoPnt(ps3, pt3)
Lsp1b = geompy.MakeLineTwoPnt(ps5, pt5)
Lsp2b = geompy.MakeLineTwoPnt(ps6, pt6)
Lsp1c = geompy.MakeLineTwoPnt(ps8, pt8)
Lsp2c = geompy.MakeLineTwoPnt(ps9, pt9)
Lsp1d = geompy.MakeLineTwoPnt(ps11, pt11)
Lsp2d = geompy.MakeLineTwoPnt(ps12, pt12)



Li1 = geompy.MakeLineTwoPnt(pi1, pi2)
Li2 = geompy.MakeLineTwoPnt(pi2, pi3)
Li3 = geompy.MakeLineTwoPnt(pi3, pi4)
Li4 = geompy.MakeLineTwoPnt(pi4, pi1)

#define lines for partition
Lp1 = geompy.MakeLineTwoPnt(pt1, pt4 )
Lp2 = geompy.MakeLineTwoPnt(pt4, pt7 )
Lp3 = geompy.MakeLineTwoPnt(pt7, pt10)
Lp4 = geompy.MakeLineTwoPnt(pt10,pt1 )
Lw1 = geompy.MakeWire([Lp1, Lp2, Lp3, Lp4])  # wire for partitioning of two faces



#define face between lines
#Face1 = geompy.MakeFaceWires([L1a, L2a, L3a, L4a], 1)
shell = geompy.MakeFaceWires([L1a, L2a, L3a, L4a, L1b, L3b, L4b, L6b, L7b, L9b, L10b, L12b, L1c, L3c, L4c, L6c, L7c, L9c, L10c, L12c, Lsp1a, Lsp2a, Lsp1b, Lsp2b, Lsp1c, Lsp2c, Lsp1d, Lsp2d, Li1, Li2, Li3, Li4], 1)

# partition face1 into shellin  and shellout for different wall thickness
PTshell = geompy.MakePartition([shell], [Lw1], [], [], geompy.ShapeType["FACE"], 0, [], 0)
listSubShapeIDs = geompy.SubShapeAllIDs(PTshell, geompy.ShapeType["FACE"])
shellout = geompy.CreateGroup(PTshell, geompy.ShapeType["FACE"])
geompy.UnionIDs(shellout, [2])
PTshell = geompy.GetMainShape(shellout)
shellin = geompy.CreateGroup(PTshell, geompy.ShapeType["FACE"])
geompy.UnionIDs(shellin, [53])
PTshell = geompy.GetMainShape(shellin)

# define groups for adding constraints and forces

outedge = geompy.CreateGroup(PTshell, geompy.ShapeType["EDGE"])
geompy.UnionIDs(outedge, [4, 9, 11, 7])
PTshell = geompy.GetMainShape(outedge)
inedge = geompy.CreateGroup(PTshell, geompy.ShapeType["EDGE"])
geompy.UnionIDs(inedge, [73, 75, 71, 68])
PTshell = geompy.GetMainShape(inedge)

node1 = geompy.CreateGroup(PTshell, geompy.ShapeType["VERTEX"])
geompy.UnionIDs(node1, [74])
PTshell = geompy.GetMainShape(node1)
nodes13 = geompy.CreateGroup(PTshell, geompy.ShapeType["VERTEX"])
geompy.UnionIDs(nodes13, [69, 74])
PTshell = geompy.GetMainShape(nodes13)
nodes4 = geompy.CreateGroup(PTshell, geompy.ShapeType["VERTEX"])
geompy.UnionIDs(nodes4, [74, 72, 69, 70])
PTshell = geompy.GetMainShape(nodes4)



# add all the components to the study
geompy.addToStudy( p1, "p1" )
geompy.addToStudy( p2, "p2" )
geompy.addToStudy( p3, "p3" )
geompy.addToStudy( p4, "p4" )
geompy.addToStudy( ps1, "ps1" )
geompy.addToStudy( ps2, "ps2" )
geompy.addToStudy( ps3, "ps3" )
geompy.addToStudy( ps4, "ps4" )
geompy.addToStudy( ps5, "ps5" )
geompy.addToStudy( ps6, "ps6" )
geompy.addToStudy( ps7, "ps7" )
geompy.addToStudy( ps8, "ps8" )
geompy.addToStudy( ps9, "ps9" )
geompy.addToStudy( ps10, "ps10" )
geompy.addToStudy( ps11, "ps11" )
geompy.addToStudy( ps12, "ps12" )

geompy.addToStudy( pt1, "pt1" )
geompy.addToStudy( pt2, "pt2" )
geompy.addToStudy( pt3, "pt3" )
geompy.addToStudy( pt4, "pt4" )
geompy.addToStudy( pt5, "pt5" )
geompy.addToStudy( pt6, "pt6" )
geompy.addToStudy( pt7, "pt7" )
geompy.addToStudy( pt8, "pt8" )
geompy.addToStudy( pt9, "pt9" )
geompy.addToStudy( pt10, "pt10" )
geompy.addToStudy( pt11, "pt11" )
geompy.addToStudy( pt12, "pt12" )

geompy.addToStudy( pi1, "pi1" )
geompy.addToStudy( pi2, "pi2" )
geompy.addToStudy( pi3, "pi3" )
geompy.addToStudy( pi4, "pi4" )

geompy.addToStudy( L1a, "L1a" )
geompy.addToStudy( L2a, "L2a" )
geompy.addToStudy( L3a, "L3a" )
geompy.addToStudy( L4a, "L4a" )
geompy.addToStudy( L1b, "L1b" )
#geompy.addToStudy( L2b, "L2b" )
geompy.addToStudy( L3b, "L3b" )
geompy.addToStudy( L4b, "L4b" )
#geompy.addToStudy( L5b, "L5b" )
geompy.addToStudy( L6b, "L6b" )
geompy.addToStudy( L7b, "L7b" )
#geompy.addToStudy( L8b, "L8b" )
geompy.addToStudy( L9b, "L9b" )
geompy.addToStudy( L10b, "L10b" )
#geompy.addToStudy( L11b, "L11b" )
geompy.addToStudy( L12b, "L12b" )

geompy.addToStudy( L1c, "L1c" )
#geompy.addToStudy( L2c, "L2c" )
geompy.addToStudy( L3c, "L3c" )
geompy.addToStudy( L4c, "L4c" )
#geompy.addToStudy( L5c, "L5c" )
geompy.addToStudy( L6c, "L6c" )
geompy.addToStudy( L7c, "L7c" )
#geompy.addToStudy( L8c, "L8c" )
geompy.addToStudy( L9c, "L9c" )
geompy.addToStudy( L10c, "L10c" )
#geompy.addToStudy( L11c, "L11c" )
geompy.addToStudy( L12c, "L12c" )

geompy.addToStudy( Lsp1a, "Lsp1a" )
geompy.addToStudy( Lsp2a, "Lsp2a" )
geompy.addToStudy( Lsp1b, "Lsp1b" )
geompy.addToStudy( Lsp2b, "Lsp2b" )
geompy.addToStudy( Lsp1c, "Lsp1c" )
geompy.addToStudy( Lsp2c, "Lsp2c" )
geompy.addToStudy( Lsp1d, "Lsp1d" )
geompy.addToStudy( Lsp2d, "Lsp2d" )


geompy.addToStudy( Li1, "Li1" )
geompy.addToStudy( Li2, "Li2" )
geompy.addToStudy( Li3, "Li3" )
geompy.addToStudy( Li4, "Li4" )

geompy.addToStudy(shell , "shell" )


geompy.addToStudy( Lp3, "Lp1" )
geompy.addToStudy( Lp4, "Lp2" )
geompy.addToStudy( Lp3, "Lp3" )
geompy.addToStudy( Lp4, "Lp4" )
geompy.addToStudy( Lw1, "Lw1" )
geompy.addToStudy(PTshell,"PTshell" )

geompy.addToStudyInFather( PTshell, shellout, "shellout" )
geompy.addToStudyInFather( PTshell, shellin, "shellin" )

geompy.addToStudyInFather( PTshell, outedge, "outedge" )
geompy.addToStudyInFather( PTshell, inedge,  "inedge"  )
geompy.addToStudyInFather( PTshell, node1,   "node1"   )
geompy.addToStudyInFather( PTshell, nodes13, "nodes13" )
geompy.addToStudyInFather( PTshell, nodes4,  "nodes4"  )



#=======================
# start the mesher here

NETGEN_2D_Parameters = smesh.smesh.CreateHypothesis('NETGEN_Parameters_2D', 'NETGENEngine')
NETGEN_2D_Parameters.SetMaxSize( 0.1 )
NETGEN_2D_Parameters.SetSecondOrder( 1 )
NETGEN_2D_Parameters.SetOptimize( 1 )
NETGEN_2D_Parameters.SetFineness( 3 )
NETGEN_2D_Parameters.SetQuadAllowed( 1 )
shell = smesh.Mesh(PTshell)

status = shell.AddHypothesis(NETGEN_2D_Parameters)
Netgen_1D_2D = smesh.smesh.CreateHypothesis('NETGEN_2D', 'NETGENEngine')
status = shell.AddHypothesis(Netgen_1D_2D)
isDone = shell.Compute()

#=============================
#============================
outedge = shell.Group(outedge)
inedge  = shell.Group(inedge)
node1   = shell.Group(node1)
nodes13 = shell.Group(nodes13)
nodes4  = shell.Group(nodes4)
shin    = shell.Group(shellin)
shout   = shell.Group(shellout)

smesh.SetName(shell.GetMesh(), 'shell')
smesh.SetName(outedge, 'outedge')
smesh.SetName(inedge,  'inedge')
smesh.SetName(node1,   'node1')
smesh.SetName(nodes13, 'nodes13')
smesh.SetName(nodes4,  'nodes4')
smesh.SetName(shin, 'shin')
smesh.SetName(shout,'shout')




smesh.SetName(NETGEN_2D_Parameters, 'NETGEN_Parameters_2D')
smesh.SetName(Netgen_1D_2D, 'NETGEN_2D')

salome.sg.updateObjBrowser(0)
